// Generated by CoffeeScript 1.12.6

/*
 * User YuanXiangDong
 * Date 17-10-01
 * aliyun search main V3
 * aliyun开放式搜索的接口实现(V3)
 */

(function() {
  var Search, Tools, _, assert, debuglog, request;

  debuglog = require('debug')('ali-opensearch::search');

  assert = require("assert");

  Tools = require('./utils/tools');

  _ = require("underscore");

  request = require('request');

  Search = (function() {

    /*
     * @param options:{}
     *   accessKeyId, 必须
     *   accessKeySecret, 必须
     *   apiURL，接口域名， 必须
     *   appName 开放搜索的应用名，必须, 如果不传，则只能使用models/app 模块
     *   pageSize 单页结果数量，默认40条
     *   format   返回数据的格式，默认JSON（有xml、json、fulljson三种格式可选。）
     *   timeout 默认3000
     */
    function Search(searchOptions) {
      assert(searchOptions, "missing options");
      assert(searchOptions.accessKeyId, "missing opensearch key id");
      assert(searchOptions.accessKeySecret, "missing opensearch key secret");
      this.GET_HTTP_METHOD = "GET";
      this.POST_HTTP_METHOD = "POST";
      this.SEARCH_LIST_SIZE = 40;
      this.URI_PREFIX = '/v3/openapi/apps';
      this.CONTENT_TYPE = 'application/json';
      this.version = 'v3';
      this.accessKeyId = searchOptions.accessKeyId;
      this.accessKeySecret = searchOptions.accessKeySecret;
      this.apiURL = searchOptions.apiURL || 'http://opensearch-cn-hangzhou.aliyuncs.com';
      this.appName = searchOptions.appName;
      this.pageSize = searchOptions.pageSize || this.SEARCH_LIST_SIZE;
      this.format = searchOptions.format || 'json';
      this.timeout = searchOptions.timeout || 3000;
      require('./utils/signature').init(this.accessKeyId, this.accessKeySecret);
      this.mixin(require('./models/actions'));
      this.mixin(require('./models/searchs'));
      this.mixin(require('./models/suggest'));
      this.mixin(require('./models/apps'));
      return;
    }

    Search.prototype.mixin = function(obj) {
      var key, v;
      for (key in obj) {
        v = obj[key];
        this[key] = v;
      }
    };

    Search.prototype.apiCall = function(url, httpMethod, httpHeader, form, callback) {
      var options;
      options = {
        url: "" + url,
        method: httpMethod,
        timeout: this.timeout,
        headers: httpHeader
      };
      if (form != null) {
        options.body = form;
      }
      console.dir(options);
      request(options, (function(_this) {
        return function(err, res, body) {
          body = Tools.parseResult(body);
          callback(err, body);
        };
      })(this));
    };

    module.exports = Search;

    return Search;

  })();

}).call(this);
