// Generated by CoffeeScript 1.12.6
(function() {
  var _, crypto, debuglog, exports, generateHeader, generateNonce, generateUri, makeConfig, makeFields, makeMultipleIdStr, makeQuery, makeSummary, parseResult, urlEncode, urlUtil;

  debuglog = require('debug')('ali-opensearch::utils::tools');

  _ = require('underscore');

  crypto = require('crypto');

  urlUtil = require('url');

  urlEncode = require('./url_encode');

  Date.prototype.Format = function(fmt) {
    var k, o, val;
    o = {
      "M+": this.getMonth() + 1,
      "d+": this.getDate(),
      "h+": this.getHours(),
      "m+": this.getMinutes(),
      "s+": this.getSeconds(),
      "q+": Math.floor((this.getMonth() + 3) / 3),
      "S": this.getMilliseconds()
    };
    if (/(y+)/.test(fmt)) {
      fmt = fmt.replace(RegExp.$1, (this.getFullYear() + "").substr(4 - RegExp.$1.length));
    }
    for (k in o) {
      val = o[k];
      if (new RegExp("(" + k + ")").test(fmt)) {
        fmt = fmt.replace(RegExp.$1, RegExp.$1.length === 1 ? o[k] : ("00" + o[k]).substr(("" + o[k]).length));
      }
    }
    return fmt;
  };

  generateNonce = function(date) {
    return String(Math.floor(date.getTime() / 1000)) + String(Math.round(Math.random() * 89999 + 10000));
  };

  generateHeader = function(query, contentType, date, nonce) {
    var header;
    date || (date = new Date(new Date().getTime() - 28800000));
    header = {
      "Date": date.Format("yyyy-MM-ddThh:mm:ssZ"),
      "Content-Type": contentType || 'application/json',
      "X-Opensearch-Nonce": nonce || generateNonce(date)
    };
    return header;
  };

  generateUri = function(uri, http_params) {
    var queryString;
    queryString = urlEncode.query2string(http_params);
    if (_.isEmpty(queryString)) {
      return "" + uri;
    }
    return uri + "?" + queryString;
  };

  parseResult = function(data, page, pageSize) {
    var err, result;
    if (!data) {
      return data;
    }
    try {
      data = JSON.parse(data);
      if (page == null) {
        return data;
      }
      if (data.status !== 'OK') {
        return data;
      }
      result = data.result;
      if (!result) {
        return data;
      }
      result['page'] = page;
      result['pagetotal'] = parseInt((result.viewtotal + pageSize - 1) / pageSize);
      data.result = result;
      return data;
    } catch (error) {
      err = error;
      return data;
    }
  };

  makeMultipleIdStr = function(ids) {
    var i, len, str, val;
    if (_.isString(ids)) {
      return ids;
    }
    if (_.isArray(ids)) {
      str = "";
      for (i = 0, len = ids.length; i < len; i++) {
        val = ids[i];
        str += "id:'" + val + "' OR ";
      }
      return str.substring(0, str.length - 3);
    }
    return null;
  };

  makeFields = function(fields) {
    var i, len, str, val;
    if (_.isString(fields)) {
      return fields;
    }
    if (_.isArray(fields)) {
      str = "";
      for (i = 0, len = fields.length; i < len; i++) {
        val = fields[i];
        str += val + ";";
      }
      return str.substring(0, str.length - 1);
    }
    return null;
  };

  makeConfig = function(config, pageSize) {
    var str;
    if (_.isString(config)) {
      return config;
    }
    if (_.isObject(config)) {
      str = "format:" + (config.format || 'json') + ",start:" + (config.start || 0) + ",hit:" + (config.hit || pageSize);
      if (config.rerank_size != null) {
        str += ",rerank_size:" + config.rerank_size;
      }
      return str;
    }
    return "format:json,start:" + 0 + ",hit:" + pageSize;
  };

  makeSummary = function(summary) {
    var key, str, val;
    if (_.isString(summary)) {
      return summary;
    }
    if (_.isObject(summary)) {
      str = "";
      for (key in summary) {
        val = summary[key];
        str += key + ":" + val + ",";
      }
      return str.substring(0, str.length - 1);
    }
    return null;
  };

  makeQuery = function(querys, pageSize) {
    var query;
    if (!((querys != null) && (querys.query != null))) {
      return null;
    }
    query = "config=" + (makeConfig(querys.config, pageSize)) + "&&query=" + querys.query;
    if (querys.filter != null) {
      query += "&&filter=" + querys.filter;
    }
    if (querys.sort != null) {
      query += "&&sort=" + querys.sort;
    }
    if (querys.aggregate != null) {
      query += "&&aggegate=" + querys.aggregate;
    }
    if (querys.distinct != null) {
      query += "&&distnct=" + querys.distinct;
    }
    return query;
  };

  module.exports = exports = {
    generateNonce: generateNonce,
    generateHeader: generateHeader,
    generateUri: generateUri,
    parseResult: parseResult,
    makeMultipleIdStr: makeMultipleIdStr,
    makeFields: makeFields,
    makeSummary: makeSummary,
    makeConfig: makeConfig,
    makeQuery: makeQuery
  };

}).call(this);
